# Solve 2x2 linear equation system of 2
# Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2007/February/23
#

import fraction

# Solutions Types
(
    NoValidInput,
    _ValidInput,     # used internally only
    NoSolution,
    UniqueSolution,  # Mostly
    InfiniteSolutions
) = range(-1,4)

# and their names, via dictionary
name = {
    NoValidInput:      "No valid input",
    NoSolution:        "No solution",
    UniqueSolution:    "Unique solution",
    InfiniteSolutions: "Infinite number of Solutions"
}


class LineEq2x2:

    """Solve equation of the form:
    a1*x + b1*y = c1
    a2*x + b2*y = c2
    """
    
    def __init__(self):
        self.coef = [ 3*[None], 3*[None] ]
        self.solution_type = NoValidInput
        self.x = None
        self.y = None


    def set_coefficients(self, sabc12):
        """Set coefficients by list of 6 strings for: a1,b1,c1, a2,b2,c2
        Return None, or error message"""
        err = None
        self.solution_type = _ValidInput  # optimistic :)
        for eq in 0,1:       # Loop over two equations
            for j in 0,1,2:  # loop over a,b,c
                f = fraction.Fraction()
                s = sabc12[3 * eq + j]  # [0,1]x[0,1,2] ==>  [0,1,2,3,4,5]
                ok = f.strset(s)
                if ok:
                    self.coef[eq][j] = f
                elif err == None: # Just the 1st error...
                    err = "Illegal number %s (@ %d,%d)" % (s, eq, j)
                    self.solution_type = NoValidInput
        return err
    

    def solve(self):
        if self.solution_type == _ValidInput:
            # Move to 'human algebraic' symbols
            [a1,b1,c1] = self.coef[0]
            [a2,b2,c2] = self.coef[1]
            # sys.stderr.write("a1=%s, b1=%s,  c1=%s\n" % (a1,b1,c1))
            # sys.stderr.write("a2=%s, b2=%s,  c2=%s\n" % (a2,b2,c2))
            det = a1*b2 - a2*b1
            if det.zero():
                # Singular cases:
                if a1 * c2 == a2 * c1:
                    self.solution_type = InfiniteSolutions
                    if c1.zero():
                        if c2.zero():
                            self.x = self.y = 0
                        else:
                            self.get_smallest_solution(a2, b2, c2)
                    else:
                        self.get_smallest_solution(a1, b1, c1)
                else:
                    self.solution_type = NoSolution
            else: # normal case
                self.solution_type = UniqueSolution
                self.x = (b2*c1 - b1*c2) / det
                self.y = (a1*c2 - a2*c1) / det
        return self.solution_type
        

    def get_smallest_solution(self, a, b, c):
        # Get smallest solution for  ax+by = c  != 0
        # sys.stderr.write("get_smallest  a=%s, b=%s, c=%s\n" % (a,b,c))
        if a.zero():
            self.x = 0
            self.y = c/b
        elif b.zero():
            self.x = c/a
            self.y = 0
        else:
            d = a*a + b*b
            self.x = a*c / d
            self.y = b*c / d
        
        
if __name__ == '__main__':
    import sys
    rc = 0
    if len(sys.argv) != 1 + 6:
        sys.stderr.write("Usage: %s  a1 b1 c1  a2 b2 c2\n" % sys.argv[0])
        rc = 1
    else:
        lineEq2x2 = LineEq2x2()
        err = lineEq2x2.set_coefficients( sys.argv[1:] )
        if err != None:
            sys.stderr.write("Error: %s\n" % err)
            rc = 2
        else:
            soltyp = lineEq2x2.solve()
            sys.stdout.write(name[soltyp])
            if soltyp in (UniqueSolution, InfiniteSolutions):
                sys.stdout.write(":  x = %s     y = %s" %
                                 (lineEq2x2.x, lineEq2x2.y))
            sys.stdout.write("\n")
    sys.exit(rc)
